package atomicstryker.minions.common.entity;

import java.util.ArrayList;
import java.util.List;

import atomicstryker.astarpathing.AS_PathEntity;
import atomicstryker.astarpathing.AStarNode;
import atomicstryker.astarpathing.AStarPathPlanner;
import atomicstryker.astarpathing.AStarStatic;
import atomicstryker.astarpathing.IAStarPathedEntity;
import atomicstryker.minions.common.MinionsCore;
import atomicstryker.minions.common.jobmanager.BlockTask;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityCreature;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.EntityAISwimming;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.pathfinding.PathPoint;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.DamageSource;
import net.minecraft.util.IChatComponent;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;
import net.minecraftforge.common.ForgeChunkManager;
import net.minecraftforge.common.ForgeChunkManager.Ticket;
import net.minecraftforge.common.ForgeChunkManager.Type;

/**
 * Minion Entity class, this is where the evil magic happens
 * 
 * @author AtomicStryker
 */

public class EntityMinion extends EntityCreature implements IAStarPathedEntity, Comparable<EntityMinion>
{
    private final int pathingCooldownTicks = 10;
    public final InventoryMinion inventory;
    public final AStarPathPlanner pathPlanner;

    public EntityPlayer master;
    public boolean inventoryFull;
    public TileEntity returnChestOrInventory;
    private AS_PathEntity pathToWalkInputCache;
    public BlockPos currentTarget;
    private int currentPathNotFoundCooldownTick;
    private int pathFindingFails;
    private int currentPathingStopCooldownTick;
    private BlockTask currentTask;
    public EntityLivingBase targetEntityToGrab;
    public float workSpeed;
    private long workBoostTime;
    public boolean isStripMining;
    private long timeLastSound;
    public boolean canPickUpItems;
    private long canPickUpItemsAgainAt;
    private long despawnTime;
    private float moveSpeed;

    public boolean followingMaster;
    public boolean returningGoods;

    private Chunk lastChunk;
    private Ticket chunkLoadingTicket;

    public EntityMinion(World var1)
    {
        super(var1);
        this.field_70178_ae = true;

        this.moveSpeed = 1.2F;
        func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.225D);
        func_110148_a(SharedMonsterAttributes.field_111265_b).func_111128_a(MinionsCore.instance.minionFollowRange);

        this.pathPlanner = new AStarPathPlanner(field_70170_p, this);

        //this.getNavigator().setAvoidsWater(false);
        this.field_70714_bg.func_75776_a(0, new EntityAISwimming(this));
        this.field_70714_bg.func_75776_a(1, new MinionAIStalkAndGrab(this, this.moveSpeed));
        this.field_70714_bg.func_75776_a(2, new MinionAIFollowMaster(this, this.moveSpeed, 10.0F, 2.0F));
        this.field_70714_bg.func_75776_a(3, new MinionAIWander(this, this.moveSpeed));
        
        inventory = new InventoryMinion(this);
        inventoryFull = false;
        currentPathNotFoundCooldownTick = 0;
        pathFindingFails = 0;
        currentPathingStopCooldownTick = 0;
        workSpeed = 1.0F;
        workBoostTime = 0L;
        isStripMining = false;
        canPickUpItems = true;
        canPickUpItemsAgainAt = 0L;
        despawnTime = -1l;

        chunkLoadingTicket = ForgeChunkManager.requestTicket(MinionsCore.instance, field_70170_p, Type.ENTITY);
        if (chunkLoadingTicket != null)
        {
            lastChunk = field_70170_p.func_175726_f(new BlockPos((int) field_70165_t, 0, (int) field_70161_v));
            chunkLoadingTicket.bindEntity(this);
            ForgeChunkManager.forceChunk(chunkLoadingTicket, lastChunk.func_76632_l());
        }
        else
        {
            System.err.println("Minions Minion "+this+" did not get a ForgeChunkManager ticket???");
        }
        
        currentTarget = BlockPos.field_177992_a;
    }

    public EntityMinion(World world, EntityPlayer playerEnt)
    {
        this(world);
        master = playerEnt;
        setMasterUserName(playerEnt.func_146103_bH().getName());
    }

    @Override
    protected void func_70088_a()
    {
        super.func_70088_a();
        /* boolean isWorking for SwingProgress and Sounds, set by AS_BlockTask */
        this.field_70180_af.func_75682_a(16, new Integer(0));
        this.field_70180_af.func_75682_a(17, new Integer(0)); // x blocktask
        this.field_70180_af.func_75682_a(18, new Integer(0)); // y blocktask
        this.field_70180_af.func_75682_a(19, new Integer(0)); // z blocktask
        this.field_70180_af.func_75682_a(20, "undef"); // masterUserName
        this.field_70180_af.func_75682_a(21, new Integer(0)); // heldItem Index
    }

    public void setWorking(boolean b)
    {
        if (!field_70170_p.field_72995_K)
        {
            field_70180_af.func_75692_b(16, b ? 1 : 0);
        }
    }

    public void setMasterUserName(String name)
    {
        if (!field_70170_p.field_72995_K)
        {
            field_70180_af.func_75692_b(20, name);
        }
    }

    public String getMasterUserName()
    {
        String s = field_70180_af.func_75681_e(20);
        return s.equals("") ? "undef" : s;
    }

    public void giveTask(BlockTask input, boolean dontReturn)
    {
        if (dontReturn)
        {
            currentTask = input;
            returningGoods = followingMaster = false;
        }
        else
        {
            currentTask = input;
            returningGoods = true;
        }
    }

    public BlockTask getCurrentTask()
    {
        return currentTask;
    }

    public boolean hasTask()
    {
        return currentTask != null;
    }

    @Override
    public boolean func_70067_L()
    {
        return true;
    }

    @Override
    public boolean func_70104_M()
    {
        return true;
    }

    @Override
    protected boolean func_70692_ba()
    {
        return false;
    }

    @Override
    public void func_70106_y()
    {
        inventory.dropAllItems();
        super.func_70106_y();
    }

    @Override
    public void func_70014_b(NBTTagCompound var1)
    {
        super.func_70014_b(var1);
        var1.func_74782_a("MinionInventory", this.inventory.writeToNBT(new NBTTagList()));
        var1.func_74778_a("masterUsername", getMasterUserName());
    }

    @Override
    public void func_70037_a(NBTTagCompound var1)
    {
        super.func_70037_a(var1);
        NBTTagList var2 = var1.func_150295_c("MinionInventory", inventory.func_70302_i_());
        this.inventory.readFromNBT(var2);
        setMasterUserName(var1.func_74779_i("masterUsername"));
        master = field_70170_p.func_72924_a(getMasterUserName());

        MinionsCore.instance.minionLoadRegister(this);
        
        currentTarget = new BlockPos((int)field_70165_t, (int)field_70163_u, (int)field_70161_v);
    }

    public void performTeleportToTarget()
    {
        if (currentTarget != null)
        {
            this.func_70634_a(currentTarget.func_177958_n() + 0.5D, currentTarget.func_177956_o(), currentTarget.func_177952_p() + 0.5D);
            MinionsCore.instance.sendSoundToClients(this, "mob.endermen.portal");
        }
    }

    public void performRecallTeleportToMaster()
    {
        if (master != null)
        {
            this.func_70634_a(master.field_70165_t + 1, master.field_70163_u, master.field_70161_v + 1);
            MinionsCore.instance.sendSoundToClients(this, "mob.endermen.portal");
        }
    }

    /**
     * Gives a Minion a list of possible target Nodes and causes the pathplanner
     * to try and path to one of them. If pathing was underway, it is
     * interrupted.
     * 
     * @param possibles
     *            list of reachable target nodes
     * @param allowDropping
     *            whether or not drops >1 block high are allowed in the path
     */
    public void orderMinionToMoveTo(AStarNode[] possibles, boolean allowDropping)
    {
        currentTarget = new BlockPos(possibles[0].x, possibles[0].y, possibles[0].z);
        pathPlanner.getPath(doubleToInt(this.field_70165_t), doubleToInt(this.field_70163_u), doubleToInt(this.field_70161_v), possibles, allowDropping);
    }

    /**
     * Orders a Minion to pathfind towards coordinates. The Pathplanner starts
     * working with the target, if there was another path being planned, it is
     * scrapped.
     */
    public void orderMinionToMoveTo(int targetX, int targetY, int targetZ, boolean allowDropping)
    {
        currentTarget = new BlockPos(targetX, targetY, targetZ);
        pathPlanner.getPath(doubleToInt(this.field_70165_t), doubleToInt(this.field_70163_u), doubleToInt(this.field_70161_v), targetX, targetY, targetZ, allowDropping);
        // System.out.println("Minion ordered to move to ["+targetX+"|"+targetY+"|"+targetZ+"]");
    }

    @Override
    public void func_70071_h_()
    {
        super.func_70071_h_();

        Chunk curChunk = field_70170_p.func_175726_f(new BlockPos((int) field_70165_t, 0, (int) field_70161_v));
        if (chunkLoadingTicket != null && curChunk != null && lastChunk != null)
        {
            if (curChunk.field_76635_g != lastChunk.field_76635_g || curChunk.field_76647_h != lastChunk.field_76647_h)
            {
                ForgeChunkManager.unforceChunk(chunkLoadingTicket, lastChunk.func_76632_l());
                lastChunk = curChunk;
                ForgeChunkManager.forceChunk(chunkLoadingTicket, lastChunk.func_76632_l());
            }
        }

        if (this.field_70153_n != null && this.field_70153_n.equals(master) && this.func_70661_as().func_75500_f())
        {
            this.field_70177_z = this.field_70125_A = 0;
        }

        if ((master != null && master.field_70128_L) || master == null)
        {
            if (despawnTime < 0)
            {
                despawnTime = System.currentTimeMillis() + MinionsCore.instance.secondsWithoutMasterDespawn * 1000l;
            }
            else if (System.currentTimeMillis() > despawnTime)
            {
                master = null;
                dropAllItemsToWorld();
                func_70106_y();
            }
        }
        else
        {
            despawnTime = -1l;
        }

        if (this.field_70180_af.func_75679_c(16) != 0)
        {
            int x = this.field_70180_af.func_75679_c(17);
            int y = this.field_70180_af.func_75679_c(18);
            int z = this.field_70180_af.func_75679_c(19);
            Block blockID = field_70170_p.func_180495_p(new BlockPos(x, y, z)).func_177230_c();

            field_70733_aJ += (0.17F * 0.5 * workSpeed);
            if (field_70733_aJ > 1.0F)
            {
                field_70733_aJ = 0;
                field_70170_p.func_180498_a(null, 2006, new BlockPos(x, y, z), field_70146_Z.nextInt(10));
            }

            if (blockID != Blocks.field_150350_a)
            {
                long curTime = System.currentTimeMillis();
                if (curTime - timeLastSound > (500L / workSpeed))
                {
                    field_70170_p.func_72980_b(field_70165_t, field_70163_u, field_70161_v, blockID.field_149762_H.func_150498_e(), (blockID.field_149762_H.func_150497_c() + 1.0F) / 2.0F,
                            blockID.field_149762_H.func_150494_d() * 0.8F, false);
                    timeLastSound = curTime;
                }
            }
        }
        else
        {
            field_70733_aJ = 0;
        }
    }

    @Override
    public void func_70030_z()
    {
        super.func_70030_z();

        if (workBoostTime != 0L && System.currentTimeMillis() - workBoostTime > 30000L)
        {
            workBoostTime = 0L;
            this.workSpeed = 1.0F;
        }

        if (func_70661_as().func_75505_d() != null)
        {
            if (hasReachedTarget())
            {
                func_70661_as().func_75484_a(null, this.moveSpeed);
            }
            else if (func_70661_as().func_75505_d() != null && func_70661_as().func_75505_d() instanceof AS_PathEntity
                    && ((AS_PathEntity) func_70661_as().func_75505_d()).getTimeSinceLastPathIncrement() > 500L && !field_70170_p.field_72995_K)
            {
                currentPathingStopCooldownTick++;
                if (currentPathingStopCooldownTick > pathingCooldownTicks)
                {
                    // System.out.println("server path follow failed trigger!");
                    currentPathingStopCooldownTick = 0;

                    PathPoint nextUp = ((AS_PathEntity) func_70661_as().func_75505_d()).getCurrentTargetPathPoint();
                    if (nextUp != null)
                    {
                        ((AS_PathEntity) func_70661_as().func_75505_d()).advancePathIndex();
                        this.func_70634_a(nextUp.field_75839_a + 0.5, nextUp.field_75837_b + 0.5, nextUp.field_75838_c + 0.5);
                        this.field_70159_w = 0;
                        this.field_70179_y = 0;
                        pathPlanner.getPath(doubleToInt(this.field_70165_t), doubleToInt(this.field_70163_u) - 1, doubleToInt(this.field_70161_v), currentTarget.func_177958_n(),
                                currentTarget.func_177956_o(), currentTarget.func_177952_p(), false);
                    }
                    else
                    {
                        performTeleportToTarget();
                    }
                }
            }
        }
        else if (returningGoods && !followingMaster)
        {
            runInventoryDumpLogic();
        }
    }

    public void runInventoryDumpLogic()
    {
        if (returnChestOrInventory == null)
        {
            if (master != null && !func_70781_l())
            {
                if (this.func_70032_d(master) < 2F && this.inventory.containsItems())
                {
                    dropAllItemsToWorld();
                    returningGoods = false;
                    func_70661_as().func_75484_a(null, this.moveSpeed);
                }
            }
        }
        else
        {
            if (this.getDistanceToTileEntity(returnChestOrInventory) > 4D)
            {
                if (!func_70781_l() || pathPlanner.isBusy())
                {
                    if (currentPathNotFoundCooldownTick > 0)
                    {
                        currentPathNotFoundCooldownTick--;
                    }
                    else
                    {
                        AStarNode[] possibles =
                                AStarStatic.getAccessNodesSorted(field_70170_p,
                                        returnChestOrInventory.func_174877_v().func_177958_n(), returnChestOrInventory.func_174877_v().func_177956_o(), returnChestOrInventory.func_174877_v().func_177952_p());
                        if (possibles.length != 0)
                        {
                            orderMinionToMoveTo(possibles, false);
                        }
                    }
                }
            }
            else
            {
                if (this.inventory.containsItems() && checkReturnChestValidity())
                {
                    this.inventory.putAllItemsToInventory((IInventory) returnChestOrInventory);
                }
                returningGoods = false;
                func_70661_as().func_75484_a(null, this.moveSpeed);
            }
        }
    }

    private boolean checkReturnChestValidity()
    {
        TileEntity test = field_70170_p.func_175625_s(returnChestOrInventory.func_174877_v());
        if (test != null)
        {
            returnChestOrInventory = test;
            return true;
        }

        returnChestOrInventory = null;
        return false;
    }

    @Override
    public void func_70636_d()
    {
        super.func_70636_d();

        if (canPickUpItems)
        {
            List<Entity> collidingEntities = this.field_70170_p.func_72839_b(this, this.func_174813_aQ().func_72314_b(1.0D, 0.0D, 1.0D));

            if (collidingEntities != null && collidingEntities.size() > 0)
            {
                for (int i = collidingEntities.size() - 1; i >= 0; i--)
                {
                    Entity ent = collidingEntities.get(i);
                    if (!ent.field_70128_L)
                    {
                        onCollisionWithEntity(ent);
                    }
                }
            }
        }
        else if (System.currentTimeMillis() > canPickUpItemsAgainAt)
        {
            canPickUpItems = true;
        }
    }

    private void onCollisionWithEntity(Entity collider)
    {
        if (collider instanceof EntityItem && !field_70170_p.field_72995_K)
        {
            EntityItem itemEnt = (EntityItem) collider;

            if (itemEnt.func_92059_d() != null)
            {
                if (itemEnt.field_70173_aa > 200)
                {
                    if (this.inventory.addItemStackToInventory(itemEnt.func_92059_d()))
                    {
                        collider.func_70106_y();
                    }
                    else
                    {
                        this.inventoryFull = true;
                        this.field_70170_p.func_72838_d(new EntityItem(field_70170_p, this.field_70165_t, this.field_70163_u, this.field_70161_v, itemEnt.func_92059_d()));
                    }
                }
            }
        }
    }

    public void dropAllItemsToWorld()
    {
        blockItemPickUp();
        MinionsCore.instance.sendSoundToClients(this, "minions:foryou");
        if (master != null)
        {
            this.func_70625_a(master, 180F, 180F);
        }
        blockItemPickUp();
        this.inventory.dropAllItems();
    }

    private void blockItemPickUp()
    {
        canPickUpItems = false;
        canPickUpItemsAgainAt = System.currentTimeMillis() + 3000L;
    }

    private double getDistanceToTileEntity(TileEntity tileent)
    {
        return AStarStatic.getDistanceBetweenCoords(doubleToInt(this.field_70165_t), doubleToInt(this.field_70163_u), doubleToInt(this.field_70161_v), tileent.func_174877_v().func_177958_n(),
                tileent.func_174877_v().func_177956_o(), tileent.func_174877_v().func_177952_p());
    }

    public boolean hasReachedTarget()
    {
        return (!func_70781_l() && currentTarget != null && AStarStatic.getDistanceBetweenCoords(doubleToInt(this.field_70165_t), doubleToInt(this.field_70163_u),
                doubleToInt(this.field_70161_v), currentTarget.func_177958_n(), currentTarget.func_177956_o(), currentTarget.func_177952_p()) < 1.5D);
    }

    @Override
    public void func_70619_bc()
    {
        if (pathToWalkInputCache != null)
        {
            // System.out.println("server updateEntActionState: Path being input!");
            this.func_70661_as().func_75484_a(pathToWalkInputCache, this.moveSpeed);
            pathToWalkInputCache = null;
        }

        if (this.hasTask())
        {
            currentTask.onUpdate();
        }

        super.func_70619_bc();
    }

    private long timelastSqueak = 0L;
    private long timeSqueakIntervals = 1000L;

    @Override
    public boolean func_70097_a(DamageSource var1, float var2)
    {
        if (this.field_70153_n != null)
        {
            this.field_70153_n.func_70078_a(null);
            return true;
        }

        if (var1.func_76346_g() != null && timelastSqueak + timeSqueakIntervals < System.currentTimeMillis())
        {
            timelastSqueak = System.currentTimeMillis();
            if (master != null && var1.func_76346_g().func_145782_y() == master.func_145782_y())
            {
                workBoostTime = System.currentTimeMillis();
                workSpeed = 2.0F;

                master.func_71009_b(this);
                MinionsCore.instance.sendSoundToClients(this, "minions:minionsqueak");
                // worldObj.playSoundAtEntity(this, "minions:minionsqueak",
                // 1.0F, 1.0F);
                return true;
            }
            else if (var1.func_76346_g() instanceof EntityPlayer)
            {
                if (this.field_70153_n != null)
                {
                    this.field_70153_n.func_70078_a(null);
                    return true;
                }
            }
        }

        return false;
    }

    public void faceBlock(int ix, int iy, int iz)
    {
        double diffX = ix - this.field_70165_t;
        double diffZ = iz - this.field_70161_v;
        double diffY = iy - this.field_70163_u;

        double var14 = (double) MathHelper.func_76133_a(diffX * diffX + diffZ * diffZ);
        float var12 = (float) (Math.atan2(diffZ, diffX) * 180.0D / 3.1415927410125732D) - 90.0F;
        float var13 = (float) (-(Math.atan2(diffY, var14) * 180.0D / 3.1415927410125732D));
        this.field_70125_A = -var13;
        this.field_70177_z = var12;
    }

    @Override
    public void onFoundPath(ArrayList<AStarNode> result)
    {
        currentPathNotFoundCooldownTick = pathingCooldownTicks;
        pathFindingFails = 0;

        pathToWalkInputCache = AStarStatic.translateAStarPathtoPathEntity(result);
        // System.out.println("Path found and translated!");

        setWorking(false);
    }

    @Override
    public void onNoPathAvailable()
    {
        if (hasTask())
        {
            currentTask.onWorkerPathFailed();
        }

        currentPathNotFoundCooldownTick = pathingCooldownTicks;
        pathFindingFails++;

        if (pathFindingFails == 3)
        {
            performTeleportToTarget();
            pathFindingFails = 0;
        }

        setWorking(false);
    }

    public IChatComponent func_145748_c_()
    {
        // return ""+(Math.sqrt((this.motionX * this.motionX) + (this.motionZ *
        // this.motionZ)));
        // return ""+currentState+"/"+nextState;
        return null;
    }

    @Override
    public ItemStack func_70694_bm()
    {
        return HeldItem.values()[field_70180_af.func_75679_c(21)].item;
    }

    private enum HeldItem
    {
        Axe(new ItemStack(Items.field_151036_c, 1)), Pickaxe(new ItemStack(Items.field_151035_b, 1)), Shovel(new ItemStack(Items.field_151037_a, 1));

        final ItemStack item;

        HeldItem(ItemStack i)
        {
            item = i;
        }
    }

    public void setHeldItemAxe()
    {
        if (!field_70170_p.field_72995_K)
        {
            field_70180_af.func_75692_b(21, HeldItem.Axe.ordinal());
        }
    }

    public void setHeldItemPickaxe()
    {
        if (!field_70170_p.field_72995_K)
        {
            field_70180_af.func_75692_b(21, HeldItem.Pickaxe.ordinal());
        }
    }

    public void setHeldItemShovel()
    {
        if (!field_70170_p.field_72995_K)
        {
            field_70180_af.func_75692_b(21, HeldItem.Shovel.ordinal());
        }
    }

    public void adaptItem(Material mat)
    {
        if (mat == Material.field_151571_B || mat == Material.field_151577_b || mat == Material.field_151578_c || mat == Material.field_151595_p || mat == Material.field_151597_y
                || mat == Material.field_151583_m)
        {
            setHeldItemShovel();
        }
        else if (mat == Material.field_151570_A || mat == Material.field_151580_n || mat == Material.field_151584_j || mat == Material.field_151585_k || mat == Material.field_151582_l
                || mat == Material.field_151569_G || mat == Material.field_151575_d)
        {
            setHeldItemAxe();
        }
        else
        {
            setHeldItemPickaxe();
        }
        // System.out.println("Minion adapted Item: "+heldItem);
    }

    public void dropMinionItemWithRandomChoice(ItemStack stack)
    {
        if (stack != null)
        {
            EntityItem itemEnt = new EntityItem(this.field_70170_p, this.field_70165_t, this.field_70163_u - 0.3D + (double) this.func_70047_e(), this.field_70161_v, stack);
            itemEnt.func_174869_p();
            float varFloatA = 0.1F;
            itemEnt.field_70159_w =
                    (double) (-MathHelper.func_76126_a(this.field_70177_z / 180.0F * 3.1415927F) * MathHelper.func_76134_b(this.field_70125_A / 180.0F * 3.1415927F) * varFloatA);
            itemEnt.field_70179_y =
                    (double) (MathHelper.func_76134_b(this.field_70177_z / 180.0F * 3.1415927F) * MathHelper.func_76134_b(this.field_70125_A / 180.0F * 3.1415927F) * varFloatA);
            itemEnt.field_70181_x = (double) (-MathHelper.func_76126_a(this.field_70125_A / 180.0F * 3.1415927F) * varFloatA + 0.1F);
            float randomAngle = this.field_70146_Z.nextFloat() * 3.1415927F * 2.0F;
            varFloatA = this.field_70146_Z.nextFloat() * 0.02F;
            itemEnt.field_70159_w += Math.cos((double) randomAngle) * (double) varFloatA;
            itemEnt.field_70181_x += (double) ((this.field_70146_Z.nextFloat() - this.field_70146_Z.nextFloat()) * 0.1F);
            itemEnt.field_70179_y += Math.sin((double) randomAngle) * (double) varFloatA;
            this.field_70170_p.func_72838_d(itemEnt);
        }
    }

    public int doubleToInt(double input)
    {
        return AStarStatic.getIntCoordFromDoubleCoord(input);
    }

    @Override
    public int compareTo(EntityMinion mother)
    {
        if (mother.func_145782_y() == this.func_145782_y())
        {
            return 0;
        }
        if (mother.func_145782_y() > this.func_145782_y())
        {
            return -1;
        }
        return 1;
    }

}
